﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Collections;
using System.IO;

namespace FM4500 {
    public partial class FOptions : Form {

        private ArrayList _listeFavoriten = new ArrayList();
        private ArrayList _listeProgs = new ArrayList();
        private ArrayList _listeLinks = new ArrayList();

        private bool retcode = false;

        public bool Retcode {
            get { return retcode; }
        }

        public FOptions(Form parent) {
            InitializeComponent();
            this.Owner = parent;  // Important, because CenterOwner            
            this.KeyPreview = true;
            this.StartPosition = FormStartPosition.CenterParent; 
        }

        private void FOptions_Load(object sender, EventArgs e) {
            setGUI();
        }

        private void setGUI() {
            cbFontSizeMenue.Value = MyRegistry.MenuSize;
            cbFontSizeTree.Value = MyRegistry.TreeSize;
            cbFontSizeListView.Value = MyRegistry.ListViewSize;
            String path = Basis.getProgPath();

            setTable(gridF);
            setTable(gridP);
            setTable(gridL);
            Middleware.readTexte(path, Konstanten.C_FAVORITEN, true, ref _listeFavoriten);  // gridF,             
            insertListeInGrid(_listeFavoriten, gridF);
            Middleware.readTexte(path, Konstanten.C_PROGS, false, ref _listeProgs);  // gridP, 
            insertListeInGrid(_listeProgs, gridP);
            Middleware.readTexte(path, Konstanten.C_LINKS, false, ref _listeLinks);
            insertListeInGrid(_listeLinks, gridL);

            rbStrgDialog.Checked = true;
            switch (MyRegistry.CopymoveAction) {
                case Konstanten.CopyMove.Dialog: rbStrgDialog.Checked = true;
                    break;
                case Konstanten.CopyMove.Strg_Copy: rbStrgCopy.Checked = true;
                    break;
                case Konstanten.CopyMove.Strg_Move: rbStrgMove.Checked = true;
                    break;
            }
            
        }

        private void setTable(DataGridView grid) {
            grid.RowCount = 1;
            grid.ColumnCount = 2;  // einen weniger, da dynamisch addiert !!
            grid.Columns[0].Name = "Bezeichnung";
            grid.Columns[1].Name = "Pfad";
            grid.Columns[0].Width = 95;
            grid.Columns[1].Width = 360;
            grid.SelectionMode = DataGridViewSelectionMode.CellSelect;
            grid.SelectionMode = DataGridViewSelectionMode.RowHeaderSelect;
            grid.SelectionMode = DataGridViewSelectionMode.FullRowSelect;


            grid.MultiSelect = false;
            grid.ReadOnly = false;
            grid.AutoSize = true;
            grid.AllowUserToAddRows = false;
            grid.AllowUserToDeleteRows = false;
        }


        private void insertListeInGrid(ArrayList liste, DataGridView grid) {
            Favorit f;
            grid.RowCount = liste.Count;
            for (int row = 0; row < liste.Count; row++) {
                f = (Favorit)liste[row];
                grid[0, row].Value = f.Bez;
                grid[1, row].Value = f.Path;
            }
        }


        private void bnOk_Click(object sender, EventArgs e) {
            MyRegistry.MenuSize = (int)cbFontSizeMenue.Value;
            MyRegistry.TreeSize = (int)cbFontSizeTree.Value;
            MyRegistry.ListViewSize = (int)cbFontSizeListView.Value;
            String path = Basis.getProgPath();
            FileStream oFile = new FileStream(path + Konstanten.C_FAVORITEN, FileMode.Create);
            StreamWriter oStream = new StreamWriter(oFile);
            foreach (Favorit f in _listeFavoriten) {
                oStream.WriteLine(f.Bez);
                oStream.WriteLine(f.Path);
            }
            oStream.Close();
            oFile.Close();

            oFile = new FileStream(path + Konstanten.C_PROGS, FileMode.Create);
            oStream = new StreamWriter(oFile);
            foreach (Favorit f in _listeProgs) {
                oStream.WriteLine(f.Bez);
                oStream.WriteLine(f.Path);
            }
            oStream.Close();
            oFile.Close();

            if (rbStrgDialog.Checked) {
                MyRegistry.CopymoveAction = Konstanten.CopyMove.Dialog;
            }
            if (rbStrgCopy.Checked) {
                MyRegistry.CopymoveAction = Konstanten.CopyMove.Strg_Copy;
            }
            if (rbStrgMove.Checked) {
                MyRegistry.CopymoveAction = Konstanten.CopyMove.Strg_Move;
            }

          

            retcode = true;
            Close();
        }

        private void bnEsc_Click(object sender, EventArgs e) {
            Close();
        }

        private void FOptions_KeyDown(object sender, KeyEventArgs e) {
            if (e.KeyValue == 13) {
                bnOk_Click(sender, e);
            }
            if (e.KeyValue == 27) {
                bnEsc_Click(sender, e);
            }
        }



        private void EditF_Click(int row) {
            if (row >= 0 && row < _listeFavoriten.Count) {
                Favorit f = (Favorit)_listeFavoriten[row];
                FReadProg dialog = new FReadProg(this,f.Bez, f.Path);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    f.Bez = dialog.Bez;
                    f.Path = dialog.Prog;
                }
            }
        }

        private void bnEditF_Click(object sender, EventArgs e) {
            int row = gridF.CurrentCell.RowIndex;
            if (row >= 0) {
                EditF_Click(row);
            }
        }

        private void bnDeleteF_Click(object sender, EventArgs e) {
            int row = gridF.CurrentCell.RowIndex;
            if (row >= 0 && row < (_listeFavoriten.Count - 1)) {
                Favorit f = (Favorit)_listeFavoriten[row];
                if (Basis.readYesNo("Wollen Sie wirklich den Pfad löschen?\r\n" +
                    "Bez: " + f.Bez + "\r\n" +
                    "Pfad: " + f.Path + "\r\n", "Eintrag löschen")) {
                    _listeFavoriten.RemoveAt(row);
                    insertListeInGrid(_listeFavoriten, gridF);
                }
            }
        }

        private void bnTopF_Click(object sender, EventArgs e) {
            int row = gridF.CurrentCell.RowIndex;
            if (row > 0 && row < _listeFavoriten.Count) {
                Favorit f = (Favorit)_listeFavoriten[row];
                _listeFavoriten.RemoveAt(row);
                _listeFavoriten.Insert(row - 1, f);
                gridF[0, row].Value = gridF[0, row - 1].Value;
                gridF[1, row].Value = gridF[1, row - 1].Value;
                gridF[0, row - 1].Value = f.Bez;
                gridF[1, row - 1].Value = f.Path;
                gridF.Rows[row].Selected = true;
            }
        }

        private void bnDownF_Click(object sender, EventArgs e) {
            int row = gridF.CurrentCell.RowIndex;
            if (row >= 0 && row < (_listeFavoriten.Count - 1)) {
                Favorit f = (Favorit)_listeFavoriten[row];
                _listeFavoriten.RemoveAt(row);
                _listeFavoriten.Insert(row + 1, f);
                gridF[0, row].Value = gridF[0, row + 1].Value;
                gridF[1, row].Value = gridF[1, row + 1].Value;
                gridF[0, row + 1].Value = f.Bez;
                gridF[1, row + 1].Value = f.Path;
                gridF.Rows[row].Selected = true;
            }
        }


        private void gridF_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e) {
            int row = e.RowIndex;
            if (row >= 0 && row < _listeFavoriten.Count) {
                EditF_Click(row);
            }
        }

        private void bnNewF_Click(object sender, EventArgs e) {
            FReadFavorit dialog = new FReadFavorit(this, "", "");
            dialog.ShowDialog();
            if (dialog.Retcode) {
                Favorit f = new Favorit();
                f.Bez = dialog.Bez;
                f.Path = dialog.Path;
                _listeFavoriten.Add(f);
                gridF.RowCount = _listeFavoriten.Count;
                gridF[0, _listeFavoriten.Count - 1].Value = f.Bez;
                gridF[1, _listeFavoriten.Count - 1].Value = f.Path;
            }
        }

        // -------------------------

        private void bnNewP_Click(object sender, EventArgs e) {
            FReadProg dialog = new FReadProg(this, "", "");
            dialog.ShowDialog();
            if (dialog.Retcode) {
                Favorit f = new Favorit();
                f.Bez = dialog.Bez;
                f.Path = dialog.Prog;
                _listeProgs.Add(f);
                gridP.RowCount = _listeProgs.Count;
                gridP[0, _listeProgs.Count - 1].Value = f.Bez;
                gridP[1, _listeProgs.Count - 1].Value = f.Path;
            }
        }

        private void bnEditP_Click(object sender, EventArgs e) {
            int row = gridP.CurrentCell.RowIndex;
            if (row >= 0) {
                EditP_Click(row);
            }
        }

        private void gridP_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e) {
            int row = e.RowIndex;
            if (row >= 0 && row < _listeProgs.Count) {
                EditP_Click(row);
            }
        } // gridP_CellMouseDoubleClick


        private void EditP_Click(int row) {
            if (row >= 0 && row < _listeProgs.Count) {
                Favorit f = (Favorit)_listeProgs[row];
                FReadProg dialog = new FReadProg(this, f.Bez, f.Path);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    f.Bez = dialog.Bez;
                    f.Path = dialog.Prog;
                }
            }
        }  // EditP_Click

        private void bnDeleteP_Click(object sender, EventArgs e) {
            int row = gridP.CurrentCell.RowIndex;
            if (row >= 0 && row < (_listeProgs.Count - 1)) {
                Favorit f = (Favorit)_listeProgs[row];
                if (Basis.readYesNo("Wollen Sie wirklich das Programm löschen?\r\n" +
                    "Bez: " + f.Bez + "\r\n" +
                    "Programm: " + f.Path + "\r\n", "Eintrag löschen")) {
                    _listeProgs.RemoveAt(row);
                    insertListeInGrid(_listeProgs, gridP);
                }
            }
        } // bnDeleteP_Click

        private void bnTopP_Click(object sender, EventArgs e) {
            int row = gridP.CurrentCell.RowIndex;
            if (row > 0 && row < _listeProgs.Count) {
                Favorit f = (Favorit)_listeProgs[row];
                _listeProgs.RemoveAt(row);
                _listeProgs.Insert(row - 1, f);
                gridP[0, row].Value = gridP[0, row - 1].Value;
                gridP[1, row].Value = gridP[1, row - 1].Value;
                gridP[0, row - 1].Value = f.Bez;
                gridP[1, row - 1].Value = f.Path;
                gridP.Rows[row - 1].Selected = true;
            }
        } // bnTopP_Click

        private void bnDownP_Click(object sender, EventArgs e) {
            int row = gridP.CurrentCell.RowIndex;
            if (row >= 0 && row < (_listeProgs.Count - 1)) {
                Favorit f = (Favorit)_listeProgs[row];
                _listeProgs.RemoveAt(row);
                _listeProgs.Insert(row + 1, f);
                gridP[0, row].Value = gridP[0, row + 1].Value;
                gridP[1, row].Value = gridP[1, row + 1].Value;
                gridP[0, row + 1].Value = f.Bez;
                gridP[1, row + 1].Value = f.Path;
                gridP.Rows[row + 1].Selected = true;
            }
        }

        // -------------------------------
        private void bnNewL_Click(object sender, EventArgs e) {
            FReadProg dialog = new FReadProg(this, "", "");
            dialog.ShowDialog();
            if (dialog.Retcode) {
                Favorit f = new Favorit();
                f.Bez = dialog.Bez;
                f.Path = dialog.Prog;
                _listeProgs.Add(f);
                gridL.RowCount = _listeLinks.Count;
                gridL[0, _listeLinks.Count - 1].Value = f.Bez;
                gridL[1, _listeLinks.Count - 1].Value = f.Path;
            }
        }

        private void bnEditL_Click(object sender, EventArgs e) {
            int row = gridL.CurrentCell.RowIndex;
            if (row >= 0) {
                EditL_Click(row);
            }
        }

        private void gridL_CellMouseDoubleClick(object sender, DataGridViewCellMouseEventArgs e) {
            int row = e.RowIndex;
            if (row >= 0 && row < _listeLinks.Count) {
                EditL_Click(row);
            }
        } // gridP_CellMouseDoubleClick


        private void EditL_Click(int row) {
            if (row >= 0 && row < _listeLinks.Count) {
                Favorit f = (Favorit)_listeLinks[row];
                FReadProg dialog = new FReadProg(this, f.Bez, f.Path);
                dialog.ShowDialog();
                if (dialog.Retcode) {
                    f.Bez = dialog.Bez;
                    f.Path = dialog.Prog;
                }
            }
        }  // EditP_Click

        private void bnDeleteL_Click(object sender, EventArgs e) {
            int row = gridL.CurrentCell.RowIndex;
            if (row >= 0 && row < (_listeLinks.Count - 1)) {
                Favorit f = (Favorit)_listeLinks[row];
                if (Basis.readYesNo("Wollen Sie wirklich die Datei löschen?\r\n" +
                    "Bez: " + f.Bez + "\r\n" +
                    "Programm: " + f.Path + "\r\n", "Eintrag löschen")) {
                    _listeLinks.RemoveAt(row);
                    insertListeInGrid(_listeLinks, gridL);
                }
            }
        } // bnDeleteP_Click

        private void bnTopL_Click(object sender, EventArgs e) {
            int row = gridL.CurrentCell.RowIndex;
            if (row > 0 && row < _listeLinks.Count) {
                Favorit f = (Favorit)_listeLinks[row];
                _listeLinks.RemoveAt(row);
                _listeLinks.Insert(row - 1, f);
                gridL[0, row].Value = gridL[0, row - 1].Value;
                gridL[1, row].Value = gridL[1, row - 1].Value;
                gridL[0, row - 1].Value = f.Bez;
                gridL[1, row - 1].Value = f.Path;
                gridL.Rows[row - 1].Selected = true;
            }
        } // bnTopP_Click

        private void bnDownL_Click(object sender, EventArgs e) {
            int row = gridL.CurrentCell.RowIndex;
            if (row >= 0 && row < (_listeLinks.Count - 1)) {
                Favorit f = (Favorit)_listeLinks[row];
                _listeLinks.RemoveAt(row);
                _listeLinks.Insert(row + 1, f);
                gridL[0, row].Value = gridL[0, row + 1].Value;
                gridL[1, row].Value = gridL[1, row + 1].Value;
                gridL[0, row + 1].Value = f.Bez;
                gridL[1, row + 1].Value = f.Path;
                gridL.Rows[row + 1].Selected = true;
            }
        }







    }
}
