﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Diagnostics;

namespace FM4500 {
    public partial class Form1 : Form, IFileManager, IDeleteTabExecuted {

        private TreeNode _mainNodeDrives;
        private TreeNode _mainNodeFavoriten;

        private TabControl _activeTabControl=null;
        private TabPage _activeTabPage = null;
        private MyListView _focusfileTable = null;

        private MyListView _fileTableLeft=null;
        private MyListView _fileTableRight = null;

        private bool _search = false;

        private ContextMenuStrip _contextMenuTabPage = null;


        public Form1() {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e) {
            setGUI();
            setMenues();
            Basis.WriteRegInt(Konstanten.C_REG_ADR + "\\FileOp", "AnzFiles", 0);
        }

        

        private void setGUI() {
            tableLayoutPanel1.Dock = DockStyle.Fill;
            tabcontrol1.Dock = DockStyle.Fill;
            tabcontrol2.Dock = DockStyle.Fill;
            splitContainer2.Dock = DockStyle.Fill;
            tableLayoutPanel2.Dock = DockStyle.Fill;
            tableLayoutPanel3.Dock = DockStyle.Fill;


            MyRegistry.loadMyRegistry();

            TreeNode root, mainNode, node1; // System.Environment.SpecialFolder

            tree.ImageList = imageList1;
            tree.BeginUpdate(); // Unterdrücken des Neuzeichnens

            tree.Nodes.Clear();

            // Insert the roots
            root = new TreeNode("Computer", 5, 5);
            root.ForeColor = Color.Red;
            root.Tag = null;
            tree.Nodes.Add(root);


            // Insert if the roots
            mainNode = new TreeNode("User", 6, 6);
            mainNode.ForeColor = Color.Blue;
            mainNode.Tag = null;
            root.Nodes.Add(mainNode);

            node1 = new TreeNode("Desktop", 7, 7);
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
            mainNode.Nodes.Add(node1);

            node1 = new TreeNode("Dokumente", 7, 7);
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.CommonDocuments);
            mainNode.Nodes.Add(node1);

            node1 = new TreeNode("Bilder", 7, 7);
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.CommonPictures);
            mainNode.Nodes.Add(node1);

            node1 = new TreeNode("Musik", 7, 7);
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.CommonMusic);
            mainNode.Nodes.Add(node1);

            node1 = new TreeNode("Videos", 7, 7);
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.CommonVideos);
            mainNode.Nodes.Add(node1);

            node1 = new TreeNode("Download", 7, 7);
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory).Replace("Desktop", "Downloads");
            mainNode.Nodes.Add(node1);

            node1 = new TreeNode("Windows-Verz.", 7, 7);
            node1.Tag = Environment.GetFolderPath(Environment.SpecialFolder.Windows);
            mainNode.Nodes.Add(node1);

            node1 = new TreeNode("Temp-Verz.", 7, 7);
            node1.Tag = System.IO.Path.GetTempPath();
            mainNode.Nodes.Add(node1);

            _mainNodeFavoriten = new TreeNode("Favoriten", 6, 6);
            _mainNodeFavoriten.ForeColor = Color.Green;
            _mainNodeFavoriten.Tag = null;
            root.Nodes.Add(_mainNodeFavoriten);

            insertFavoriten(_mainNodeFavoriten);

            insertProgs(mainProgs);

            insertLinks(mainLinks);

            _mainNodeDrives = new TreeNode("Laufwerke", 6, 6);
            _mainNodeDrives.ForeColor = Color.Blue;
            _mainNodeDrives.Tag = null;
            root.Nodes.Add(_mainNodeDrives);

            insertDrives(_mainNodeDrives);

            tree.EndUpdate();  // Neu Zeichnen
            root.Expand();
            _mainNodeDrives.ExpandAll();


            setFonts();
            
            splitContainer2.Dock = DockStyle.Fill;

            tabcontrol1.Dock = DockStyle.Fill;
            tabcontrol2.Dock = DockStyle.Fill;
            

            String actualPath1 = Basis.ReadRegString(Konstanten.C_REG_ADR, "Folder1");
            if (!Directory.Exists(actualPath1)) actualPath1 = getFirstFolder();
            TabPage tabitem1 = new TabPage();
            MyListView fileTable1 = new MyListView(this, tabcontrol1, tabitem1, actualPath1, imageListSmall, imageListBig, new Konstanten.setFilenamePath(setFilenamePath1));
            tabitem1.Controls.Add(fileTable1);
            tabitem1.Dock = DockStyle.Fill;
            // tabitem1.Content = fileTable1;
            tabcontrol1.Controls.Add(tabitem1);
            tabitem1.Dock = DockStyle.Fill;
            fileTable1.Dock = DockStyle.Fill;

            tabitem1.Focus();

            _activeTabControl=tabcontrol1;
            _activeTabPage = tabitem1;
            _focusfileTable = fileTable1;

            _fileTableLeft=fileTable1;
            


            String actualPath2 = Basis.ReadRegString(Konstanten.C_REG_ADR, "Folder2");
            if (!Directory.Exists(actualPath2)) actualPath2 = getFirstFolder();
            TabPage tabitem2 = new TabPage();
            MyListView fileTable2 = new MyListView(this, tabcontrol2, tabitem2, actualPath2, imageListSmall, imageListBig, new Konstanten.setFilenamePath(setFilenamePath2));
            tabitem2.Controls.Add(fileTable2);
            tabitem2.Dock = DockStyle.Fill;
            fileTable2.Dock = DockStyle.Fill;
            // tabitem2.Content = fileTable2;
            tabcontrol2.Controls.Add(tabitem2);
            _fileTableRight = fileTable2;
            

            tabitem2.Focus();
            tabitem1.Focus();
            // _fileTableLeft = new MyListView(this, this.fileSystemWatcher1, fileTable1, actualPath, new Konstanten.setFilenamePath(setFilenamePath1));


        }  // setGUI

        // 
        public void setFilenamePath1(String path, String filename, TabControl tb, TabPage tp, MyListView lv) {
            if (path.Equals(Konstanten.C_TOP_DIR))
                listViewPath1.Text = "";
            else
                listViewPath1.Text = path;
            this.listViewFilename1.Text = filename;
            _activeTabControl = tb;
            _activeTabPage = tp;
            _focusfileTable = lv;
            _fileTableLeft = lv;
        }
        
        public void setFilenamePath2(String path, String filename, TabControl tb, TabPage tp, MyListView lv) {
            if (path.Equals(Konstanten.C_TOP_DIR))
                listViewPath2.Text = "";
            else
                listViewPath2.Text = path;
            this.listViewFilename2.Text = filename;
            _activeTabControl = tb;
            _activeTabPage = tp;
            _focusfileTable = lv;
            _fileTableRight = lv;
        }



        private String getFirstFolder() {
            foreach (TreeNode item in _mainNodeDrives.Nodes) {
                String path = ((String)item.Tag);
                DriveInfo drv = new DriveInfo(path);
                if (drv.IsReady) {
                    return path;
                }
            }
            return "c:\\";
        }

        private TreeNode addNode(TreeNode parent, String filename, String fullname, int sym) {
            TreeNode node;
            node = new TreeNode(filename, sym, sym);
            node.Checked = true;
            node.Tag = fullname;
            parent.Nodes.Add(node);
            return node;
        }

        private void insertDrives(TreeNode mainNodeDrives) {
            TreeNode node1;
            mainNodeDrives.Nodes.Clear();
            // String[] drives = System.Environment.GetLogicalDrives();  // get only the Strings
            DriveInfo[] drives = DriveInfo.GetDrives();
            foreach (DriveInfo drv in drives) {
                String typ = "";
                int sym = 0;
                switch (drv.DriveType) {
                    case DriveType.CDRom: typ = "CD/DVD:"; sym = 0; break;
                    case DriveType.Fixed: typ = "Platte:"; sym = 1; break;
                    case DriveType.Network: typ = "Netz:"; sym = 2; break;
                    case DriveType.Ram: typ = "Ram:"; break;
                    case DriveType.Removable: typ = "USB:"; sym = 3; break;
                    case DriveType.Unknown: typ = "?:"; break;
                }
                String filename = "";
                if (drv.DriveType != DriveType.CDRom && drv.DriveType != DriveType.Removable)
                    if (drv.VolumeLabel.Equals("")) {
                        filename = typ + " (" + drv.RootDirectory + ")";
                    }
                    else {
                        filename = drv.VolumeLabel + " (" + drv.RootDirectory + ")";
                    }
                else {
                    if (drv.DriveType == DriveType.CDRom)
                        filename = typ + " (" + drv.RootDirectory + ")";
                    else
                        filename = typ + " (" + drv.RootDirectory + ")";
                }
                node1 = addNode(mainNodeDrives, filename, drv.RootDirectory.FullName, sym);
                // mainNodeDrives.Nodes.Add(node1);
            }  // foreach
        }



        private void insertFavoriten(TreeNode parent) {
            // path = Application.StartupPath;  // nur WinForms
            TreeNode node;
            String path = Basis.getProgPath();
            parent.Nodes.Clear();

            //path = Directory.GetCurrentDirectory(); //System.Environment.CurrentDirectory;
            // path = System.Environment.CurrentDirectory;            
            if (File.Exists(path + Konstanten.C_FAVORITEN)) {
                String[] items = File.ReadAllLines(path + Konstanten.C_FAVORITEN);
                int i = 0;
                while (i < items.Length) {
                    if (items[i].Length > 0) {
                        if (!Directory.Exists(items[i + 1])) {
                            Basis.ErrorMsg("Das Verzeichnis ist nicht vorhanden\r\n" + items[i + 1], "Eintragen der Favoriten-Verzeichnisse");
                            i += 2;
                        }
                        else {
                            node = new TreeNode(items[i], 7, 7);
                            i++;
                            node.Tag = items[i];
                            parent.Nodes.Add(node);
                            i++;
                        }
                    }
                    else {
                        i++;
                        i++;
                    }
                }
            }
        }  // insertFavoriten


        private void insertProgs(ToolStripMenuItem main) {
            ToolStripMenuItem mnProg;
            String path = Basis.getProgPath();
            String filepath, name;
            main.DropDownItems.Clear();
            if (File.Exists(path + Konstanten.C_PROGS)) {
                String[] items = File.ReadAllLines(path + Konstanten.C_PROGS);
                int i = 0;
                while (i < items.Length) {
                    if (items[i].Length > 0) {
                        name = items[i];
                        i++;
                        filepath = items[i];
                        i++;
                        mnProg = new ToolStripMenuItem();
                        mnProg.Text = name;
                        mnProg.Tag = filepath;
                        main.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
                            mnProg});
                        //.Items.Add(mnProg);
                        mnProg.Click += new System.EventHandler(mnProgs_Click);


                    }
                    else {
                        i++;
                        i++;
                    }
                }
            }
        }
        private void insertLinks(ToolStripMenuItem main) {
            ToolStripMenuItem mnProg;
            String path = Basis.getProgPath();
            String filepath, name;
            main.DropDownItems.Clear();
            if (File.Exists(path + Konstanten.C_LINKS)) {
                String[] items = File.ReadAllLines(path + Konstanten.C_LINKS);
                int i = 0;
                while (i < items.Length) {
                    if (items[i].Length > 0) {
                        name = items[i];
                        i++;
                        filepath = items[i];
                        i++;
                        mnProg = new ToolStripMenuItem();
                        mnProg.Text = name;
                        mnProg.Tag = filepath;
                        main.DropDownItems.AddRange(new System.Windows.Forms.ToolStripItem[] {
                            mnProg});
                        mnProg.Click += new System.EventHandler(mnProgs_Click);
                    }
                    else {
                        i++;
                        i++;
                    }
                }
            }
        }  // InsertLinks



        private void tree_AfterSelect(object sender, TreeViewEventArgs e) {
            if (!_search) {
                tree_click(false);
            }
        }

        private void tree_click(bool sendToListView) {
            if (_search) return;
            TreeNode node1 = tree.SelectedNode;
            String path = (String)node1.Tag;
            if (path == null) {
                // Basis.ErrorMsg("Das Laufwerk ist null", "Wechseln in ein Laufwerk");
                return;
            }
            if (Directory.Exists(path)) {
                if (path.Length <= 3) {
                    DriveInfo drv = new DriveInfo(path);
                    if (!drv.IsReady) {
                        Basis.ErrorMsg("Das Laufwerk ist nicht bereit\r\n" + path, "Wechseln in ein Laufwerk");
                        return;
                    }
                }
                if (sendToListView) _focusfileTable.ActualPath = path;
            }
            insertSubDirsIntoTree(node1, false);
        }


        private void tree_DoubleClick(object sender, EventArgs e) {
            if (!_search) {
                tree_click(true);
            }
            // e.Handled = true; // abgearbeitet
        }

        // this method must call, if user delete subdirs or insert sub via Strg+C/Strg+V
        public void insertSubDirsIntoTree(TreeNode node1, bool clearSubs) {
            TreeNode node2, node3;
            if (node1 != null) {
                String path = (String)node1.Tag;
                if (node1.Tag != null && !Directory.Exists(path)) {
                    Basis.ErrorMsg("Das Verzeichnis ist nicht vorhanden\r\n" + path, "Wechseln in ein Verzeichnis");
                    return;
                }
                bool search = false;
                if (clearSubs && node1.Nodes.Count > 0) {
                    node1.Nodes.Clear();
                    search = true;
                }
                else {
                    // check, if the node hat Items, BUT node with subdirs has at first only ONE node with "?"
                    // if the node hat Items, don't search again
                    search = node1.Nodes.Count == 0;
                    if (!search) {
                        // test if item ist "?", ween search
                        node2 = (TreeNode)node1.Nodes[0];
                        if (node2.Text.Equals("?")) {
                            search = true;
                            node1.Nodes.Clear();
                        }
                    }
                    else {
                    }
                }

                if (search) {
                    try {
                        // search for subdirs
                        String[] dirs = Directory.GetDirectories(path);
                        foreach (String item in dirs) {
                            DirectoryInfo dir = new DirectoryInfo(item);
                            FileAttributes attribs = dir.Attributes;
                            if ((attribs & FileAttributes.Hidden) != FileAttributes.Hidden && (attribs & FileAttributes.System) != FileAttributes.System) {

                                node2 = addNode(node1, dir.Name, dir.FullName, 9);
                                try {
                                    String[] dirs2 = Directory.GetDirectories(dir.FullName);
                                    if (dirs2.Length > 0) {
                                        node3 = new TreeNode("?");
                                        node3.Tag = null;
                                        node2.Nodes.Add(node3);
                                    }
                                }
                                catch { }
                            }
                        }
                    }
                    catch (Exception) {
                    }
                }
                // showPath(path);  // TreeViewList, falsch, da diese Routine auch von expanded aufgerufen wird
            }
            else {
                // Title = "node1 == null";
            }
        }  // insertSubDirintoTrees



        

        private void mnNewFolder_Click(object sender, EventArgs e) {
            _focusfileTable.NewFolder();
        }

        private void mnLoadAgain_Click(object sender, EventArgs e) {
            _focusfileTable.LoadAgain();
        }

        private void mnPath2Clipboard_Click(object sender, EventArgs e) {
            _focusfileTable.Path2Clipboard();
        }


        private void mnClose_Click(object sender, EventArgs e) {
            Close();
        }


        private void mnCopy_Click(object sender, EventArgs e) {
            _focusfileTable.Copy();
        }

        private void mnCut_Click(object sender, EventArgs e) {
            _focusfileTable.Cut();
        }

        private void mnPaste_Click(object sender, EventArgs e) {
            _focusfileTable.Paste();
        }

        private void mnDelete_Click(object sender, EventArgs e) {
            deleteFiles(true);
        }
        private void deleteFiles(bool recycleBin) {
            _focusfileTable.DeleteFiles(recycleBin);
        }

        private void mnSelectAll_Click(object sender, EventArgs e) {
            _focusfileTable.SelectAllItems();
        }

        private void mnRenameFileF2_Click(object sender, EventArgs e) {
            _focusfileTable.RenameF2();
        }

        private void mnCopyFileF8_Click(object sender, EventArgs e) {
            _focusfileTable.CopyF8();
        }

        private void setMask(ToolStripMenuItem mn) {
            mnMask_stern.Checked = false;
            mnMask_txt.Checked = false;
            mnMask_ppt.Checked = false;
            mnMask_java.Checked = false;
            mnMask_doc.Checked = false;
            mnMask_xls.Checked = false;
            mnMask_pdf.Checked = false;
            mnMask_doku.Checked = false;
            mnMask_exe.Checked = false;
            mnOwnerDefined.Checked = false;
            mn.Checked = true;
        }

        public void setActualMask(Middleware.Masken mask, string ownerdef) {
            switch (mask) {
                case Middleware.Masken.doc: setMask(mnMask_doc); break;
                case Middleware.Masken.doku: setMask(mnMask_doku); break;
                case Middleware.Masken.exe: setMask(mnMask_exe); break;
                case Middleware.Masken.java: setMask(mnMask_java); break;
                case Middleware.Masken.pdf: setMask(mnMask_pdf); break;
                case Middleware.Masken.ppt: setMask(mnMask_ppt); break;
                case Middleware.Masken.Stern: setMask(mnMask_stern); break;
                case Middleware.Masken.txt: setMask(mnMask_txt); break;
                case Middleware.Masken.ownerdef: setMask(mnOwnerDefined);
                    mnOwnerDefined.Text = ownerdef;
                    break;
            }
        }

        private void mnMask_stern_Click(object sender, EventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.Stern);
            mnOwnerDefined.Text = "_selbst. def. Maske";
            setMask(mnMask_stern);
        }

        private void mnMask_txt_Click(object sender, EventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.txt);
            mnOwnerDefined.Text = "_selbst. def. Maske";
            setMask(mnMask_txt);
        }

        private void mnMask_ppt_Click(object sender, EventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.ppt);
            mnOwnerDefined.Text = "_selbst. def. Maske";
            setMask(mnMask_ppt);
        }

        private void mnMask_java_Click(object sender, EventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.java);
            mnOwnerDefined.Text = "_selbst. def. Maske";
            setMask(mnMask_java);
        }

        private void mnMask_doc_Click(object sender, EventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.doc);
            mnOwnerDefined.Text = "_selbst. def. Maske";
            setMask(mnMask_doc);
        }

        private void mnMask_xls_Click(object sender, EventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.xls);
            mnOwnerDefined.Text = "_selbst. def. Maske";
            setMask(mnMask_xls);

        }

        private void mnMask_pdf_Click(object sender, EventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.pdf);
            mnOwnerDefined.Text = "_selbst. def. Maske";
            setMask(mnMask_pdf);
        }

        private void mnMask_doku_Click(object sender, EventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.doku);
            mnOwnerDefined.Text = "_selbst. def. Maske";
            setMask(mnMask_doku);
        }

        private void mnMask_exe_Click(object sender, EventArgs e) {
            _focusfileTable.setMask(Middleware.Masken.exe);
            mnOwnerDefined.Text = "_selbst. def. Maske";
            setMask(mnMask_exe);
        }

        private void mnOwnerDefined_Click(object sender, EventArgs e) {
            OwnerDefined dialog = new OwnerDefined(this, "Selbstdefinierte Maske", "Selbstdefinierte Maske", "Masken, getrennt mit Semikolon",
                "*.doc* ; *.xls*");
            dialog.ShowDialog();
            if (dialog.Retcode) {
                _focusfileTable.OwnerDefined = dialog.TInput.Trim();
                _focusfileTable.setMask(Middleware.Masken.ownerdef);
                mnOwnerDefined.Text = dialog.TInput.Trim();
                setMask(mnOwnerDefined);
            }
        }

        private void mnExplorer_Click(object sender, EventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Explorer.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnProgs_Click(object sender, EventArgs e) {
            ToolStripMenuItem mnProg = (ToolStripMenuItem)sender;
            String file = (String)mnProg.Tag;
            if (File.Exists(file)) {
                Process Prog = new Process();
                Prog.StartInfo.FileName = file;
                Prog.StartInfo.Arguments = "";
                Prog.Start();
            }
            else {
                Basis.ErrorMsg("Die Datei ist nicht vorhanden\r\n" + file, "Aufrufen eines externen Programms");
            }
        }


        private void mnNotepad_Click(object sender, EventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Notepad.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnDosEbene_Click(object sender, EventArgs e) {
            _focusfileTable.DosEbene();
        }

        private void mnTaskManager_Click(object sender, EventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Taskmgr.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnSystemsteuerung_Click(object sender, EventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "control.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnSytemConfig_Click(object sender, EventArgs e) {
            if (File.Exists("c:\\Windows\\System32\\msconfig.exe")) {
                Process Prog = new Process();
                Prog.StartInfo.FileName = "c:\\Windows\\System32\\msconfig.exe";
                Prog.StartInfo.Arguments = "";
                Prog.Start();
            }
            else {
                Basis.ErrorMsg("Die Datei ist nicht vorhanden\r\n" + "c:\\Windows\\System32\\msconfig.exe", "Aufrufen eines externen Programms");
            }
        }

        private void mnmsinfo32_Click(object sender, EventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "msinfo32.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnRegedit_Click(object sender, EventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Regedit.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnCalc_Click(object sender, EventArgs e) {
            Process Prog = new Process();
            Prog.StartInfo.FileName = "Calc.exe";
            Prog.StartInfo.Arguments = "";
            Prog.Start();
        }

        private void mnRunEditor5000_Click(object sender, EventArgs e) {
            _focusfileTable.RunEditor5000();
        }
        private void mnRunNotepad_Click(object sender, EventArgs e) {
            _focusfileTable.RunNotepad();
        }

        private void mnRunHexEditor_Click(object sender, EventArgs e) {
            _focusfileTable.RunHexEditor();
        }



        private void bnGoToTopFolder_Click(object sender, EventArgs e) {
            _focusfileTable.GoToTopFolder();
        }

        private void bnToRoot_Click(object sender, EventArgs e) {
            _focusfileTable.GoToRoot();
        }


        private void mnOption_Click(object sender, EventArgs e) {
            FOptions dialog = new FOptions(this);
            dialog.ShowDialog();
            if (dialog.Retcode) {
                setFonts();
                insertFavoriten(_mainNodeFavoriten);
                insertProgs(mainWinProgs);
                for (int i = 0; i < tabcontrol1.Controls.Count; i++) {
                    TabPage tabitem = (TabPage)tabcontrol1.Controls[i];
                    MyListView fileTable = (MyListView)tabitem.Controls[0];
                    fileTable.setContextMenu();                   
                }
                for (int i = 0; i < tabcontrol2.Controls.Count; i++) {
                    TabPage tabitem = (TabPage)tabcontrol2.Controls[i];
                    MyListView fileTable = (MyListView)tabitem.Controls[0];
                    fileTable.setContextMenu();                    
                }
            }
        }

        private void mnProgInfo_Click(object sender, EventArgs e) {
            FProgInfo dialog = new FProgInfo(this);
            dialog.ShowDialog();
        }

        private void setFonts() {
            Font font = new Font("Segoe UI", MyRegistry.MenuSize);
            mainFile.Font = Font;
            mainFile.Font = font;
            mainEdit.Font = font;
            mainFile.Font = font;
            mainWinProgs.Font = font;
            this.mainOptions.Font = font;
            this.mainRun.Font = font;
            this.mainProgs.Font = font;
            this.mainMask.Font = font;
            this.mainLinks.Font = font;
            mainHelp.Font = font;

            bnLoadAgain.Font = font;
            bnRenameF2.Font = font;
            bnCopyFileF8.Font = font;

            font = new Font("Segoe UI", MyRegistry.TreeSize);
            tree.Font = font;

            font = new Font("Microsoft Sans Serif", MyRegistry.ListViewSize);
            listViewPath1.Font = font;
            listViewFilename1.Font = font;
            listViewPath2.Font = font;
            listViewFilename2.Font = font;
            tabcontrol1.Font = font;
            tabcontrol2.Font = font;
            for (int i = 0; i < tabcontrol1.Controls.Count; i++) {
                TabPage tabitem = (TabPage)tabcontrol1.Controls[i];
                MyListView fileTable = (MyListView)tabitem.Controls[0];
                fileTable.Font = font;
            }
            for (int i = 0; i < tabcontrol2.Controls.Count; i++) {
                TabPage tabitem = (TabPage)tabcontrol2.Controls[i];
                MyListView fileTable = (MyListView)tabitem.Controls[0];
                fileTable.Font = font;
            }
        }  // setFonts



        private void tree_AfterExpand(object sender, TreeViewEventArgs e) {
            if (!_search) {
                TreeNode node1 = e.Node;
                if (node1 != null) {
                    insertSubDirsIntoTree(node1, false);
                }
            }
        }

        public void insertSubDirsIntoTree(string path, bool clearSubs) {
            // throw new NotImplementedException();  
        }


        private void Form1_KeyDown(object sender, KeyEventArgs e) {
            if (!e.Alt && !e.Control && !e.Shift && e.KeyValue == 13) {
                _focusfileTable.MMouseDoubleClick();
            }
        }

        private void fileTable1_KeyDown(object sender, KeyEventArgs e) {
            // entf-Taste: delete
            if (!e.Alt && !e.Control && e.Shift && e.KeyValue == 46) {
                deleteFiles(false);
            }
            if (!e.Alt && !e.Control && !e.Shift && e.KeyValue == 46) {
                deleteFiles(true);
            }
        }

        private void fileTable2_KeyDown(object sender, KeyEventArgs e) {
            // delete
            if (!e.Alt && !e.Control && e.Shift && e.KeyValue == 46) {
                deleteFiles(false);
            }
            if (!e.Alt && !e.Control && !e.Shift && e.KeyValue == 46) {
                deleteFiles(true);
            }
        }


        
        private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
            Basis.WriteRegString(Konstanten.C_REG_ADR, "Folder1", _fileTableLeft.ActualPath);
            Basis.WriteRegString(Konstanten.C_REG_ADR, "Folder2", _fileTableRight.ActualPath);
        }


        public void DeleteTabExecuted(TabControl tabcontrol) {            
            if (tabcontrol.Controls.Count > 1) {
                tabcontrol.Controls.RemoveAt(tabcontrol.SelectedIndex);
            }
        }  //DeleteTabExecuted


        private void mnDeleteRegister_Click(object sender, EventArgs e) {
            if (_activeTabControl.Controls.Count > 1) {
                TabPage tabPage1 = (TabPage)_activeTabControl.Controls[_activeTabControl.SelectedIndex];
                _activeTabControl.Controls.RemoveAt(_activeTabControl.SelectedIndex);
            }
        }


        private void NewRegisterWithPath(TabControl tabcontrol, String path) {
                MyListView fileTable;
                if (path != null) {
                    TabPage tabitem1 = new TabPage();
                    if (_activeTabControl==tabcontrol1)
                        fileTable = new MyListView(this, tabcontrol, tabitem1, path, imageListSmall, imageListBig, new Konstanten.setFilenamePath(setFilenamePath1));
                    else
                        fileTable = new MyListView(this, tabcontrol, tabitem1, path, imageListSmall, imageListBig, new Konstanten.setFilenamePath(setFilenamePath2));
                    tabitem1.Controls.Add(fileTable);
                    tabitem1.Dock = DockStyle.Fill;
                    // tabitem1.Content = fileTable1;
                    tabcontrol.Controls.Add(tabitem1);
                    tabitem1.Dock = DockStyle.Fill;
                    fileTable.Dock = DockStyle.Fill;

                    tabitem1.Focus();
                    _activeTabPage = tabitem1;
                    _focusfileTable = fileTable;
                }
        }
        
        private void lk_newRegisterWithPath_Click(object sender, EventArgs e) {
            TreeNode node1 = tree.SelectedNode;
            String path = (String)node1.Tag;
            if (path != null  && Directory.Exists(path) ) {
                NewRegisterWithPath(_activeTabControl, path);
            }

        }

        private void mnNewRegister_Click(object sender, EventArgs e) {
            MyListView fileTable;
            String path = getFirstFolder();
            if (path != null) {
                TabPage tabitem1 = new TabPage();
                if (_activeTabControl == tabcontrol1)
                    fileTable = new MyListView(this, _activeTabControl, tabitem1, path, imageListSmall, imageListBig, new Konstanten.setFilenamePath(setFilenamePath1));
                else
                    fileTable = new MyListView(this, _activeTabControl, tabitem1, path, imageListSmall, imageListBig, new Konstanten.setFilenamePath(setFilenamePath2));
                tabitem1.Controls.Add(fileTable);
                tabitem1.Dock = DockStyle.Fill;
                _activeTabControl.Controls.Add(tabitem1);
                tabitem1.Dock = DockStyle.Fill;
                fileTable.Dock = DockStyle.Fill;

                tabitem1.Focus();
                _activeTabPage = tabitem1;
                _focusfileTable = fileTable;
            }

        }

        private void lk_newRegister_Click(object sender, EventArgs e) {
            mnNewRegister_Click(sender, e);
        }

        private void lk_NewRegisterLeftWithPath_Click(object sender, EventArgs e) {
            TreeNode node1 = tree.SelectedNode;
            String path = (String)node1.Tag;
            if (path != null && Directory.Exists(path)) {
                NewRegisterWithPath(tabcontrol1, path);
            }
        }

        private void lk_NewRegisterRightWithPath_Click(object sender, EventArgs e) {
            TreeNode node1 = tree.SelectedNode;
            String path = (String)node1.Tag;
            if (path != null && Directory.Exists(path)) {
                NewRegisterWithPath(tabcontrol2, path);
            }
        }

        private void mnLoadDrivesAgain_Click(object sender, EventArgs e) {
            insertDrives(_mainNodeDrives);
        }

        private void mnShowAllFiles_Click(object sender, EventArgs e) {
            _focusfileTable.ShowAllFiles();
        }

        private void mnCompareFolders_Click(object sender, EventArgs e) {
            _fileTableLeft.CompareFolders(_fileTableRight.ActualPath, _fileTableRight.Fileliste);
        }
        private void mnShowDriveSpace_Click(object sender, EventArgs e) {
            FShowDriveSpace dialog = new FShowDriveSpace(this, "");
            dialog.ShowDialog();
        }

        public void showDriveInfo(String drive) {
            FShowDriveSpace dialog = new FShowDriveSpace(this, drive);
            dialog.ShowDialog();
        }

        private void setMenues() {
            _contextMenuTabPage = new ContextMenuStrip();
            ToolStripMenuItem mnProg = new ToolStripMenuItem();
            mnProg.Text = "Löschen des Registers";
            mnProg.Tag = 1;
            _contextMenuTabPage.Items.Add(mnProg);
            _contextMenuTabPage.Items.Add(new ToolStripSeparator());

            mnProg = new ToolStripMenuItem();
            mnProg.Text = "Neues Register";
            mnProg.Tag = 2;
            _contextMenuTabPage.Items.Add(mnProg);
            _contextMenuTabPage.Items.Add(new ToolStripSeparator());            //ToolStripMenuItem mnMainFavoriten = new ToolStripMenuItem();
            //mnMainFavoriten.Text = "Wechsel zu Favoriten";
            //mnMainFavoriten.Tag = null;
            //_contextMenuTabPage.Items.Add(mnMainFavoriten);

            setContextMenu(Basis.getProgPath() + Konstanten.C_FAVORITEN, _contextMenuTabPage);

            mnProg.Click += new System.EventHandler(contextMenu_Click);
        }

        public void setContextMenu(String filename, ContextMenuStrip mnMain) {
            ToolStripMenuItem mnItem;
            String filepath, name;
            // mnMain.DropDownItems.Clear();
            if (File.Exists(filename)) {
                String[] items = File.ReadAllLines(filename);
                int i = 0;
                while (i < items.Length) {
                    if (items[i].Length > 0) {
                        mnItem = new ToolStripMenuItem();
                        name = items[i];
                        i++;
                        filepath = items[i];
                        i++;
                        mnItem.Text = name;
                        mnItem.Tag = filepath;
                        mnMain.Items.Add(mnItem);
                        mnItem.Click += favoriten_ctm_Click;
                    }
                    else {
                        i++;
                        i++;
                    }
                }
            }
        }  // setContextMenu

        private void favoriten_ctm_Click(object sender, EventArgs e) {
            ToolStripMenuItem mnProg = (ToolStripMenuItem)sender;
            String path = (String)mnProg.Tag;
            if (Directory.Exists(path)) {
                _focusfileTable.ActualPath = path;
            }
        }



        private void contextMenu_Click(object sender, EventArgs e) {
            ToolStripMenuItem mnItem = (ToolStripMenuItem)sender;
            int nr = (int)mnItem.Tag;
            switch (nr) {
                case 1: DeleteTabExecuted(_activeTabControl);
                    break;
                case 2: mnNewRegister_Click(sender, e);
                    break;
            }
        }
        private void tabcontrol2_MouseClick(object sender, MouseEventArgs e) {
            if (e.Button == MouseButtons.Right) {
                this._contextMenuTabPage.Show(this.tabcontrol2, e.Location);
            }
        }

        private void tabcontrol1_MouseClick(object sender, MouseEventArgs e) {
            if (e.Button == MouseButtons.Right) {
                this._contextMenuTabPage.Show(this.tabcontrol1, e.Location);
            }
        }


    }
}
